//createnote.js
//Tom Turton 2008
//
//generate a pitch (7) based on note type, random factor, previous note type and previous pitch


//Inlets and Outlets
inlets = 2;	//note type, random factor
outlets = 1;	//note pitch (7)

//Constants
const MINPITCH = 31;
const MAXPITCH = 46;

//Global vars
var type = -1;
var randomfactor = 0;

var lasttype = -1;
var lastpitch = -1;

var rangeweight;
var noteweight;

var pitch = -1;


//Functions
function msg_int(n)
{
    if(inlet == 0)    type = n;
    else if(inlet == 1)    randomfactor = n/100;
    
    if(type > 0)    doit();
}


function doit()
{
    calculateprobs();
    findpitch();
    
    if(pitch > -1)
    {
        post(rangeweight);
        post("\n");
        post(noteweight);
        post("\n");
        post(pitch);
        post("\n\n");
        
        lasttype = type;
        lastpitch = pitch;
        
        outlet(0, pitch);
        
        type = -1;
        pitch = -1;
    }
}


function calculateprobs()
{
    if(lastpitch < MINPITCH)    lastpitch = 35;
    
    rangeweight = new Array();
    noteweight = new Array();
    
    for(i=MINPITCH; i<MAXPITCH; i++)
    {
        
        //Calculate proximity probabilities from previous note type and previous note
        rangeweight[i-MINPITCH] = 10-(Math.abs(lastpitch-i));
        if(i == lastpitch)    rangeweight[i-MINPITCH] = 7;
        if((lasttype == 2) || (lasttype == 3))    if(Math.abs(lastpitch-i) != 1)    rangeweight[i-MINPITCH] = 0;
        rangeweight[i-MINPITCH] = (randomfactor * Math.random()) + ((1-randomfactor) * rangeweight[i-MINPITCH]);
        
        
        //Calculate tonal probabilities from note type
        switch(type)
        {
        case 0:	//note off
            noteweight[i-MINPITCH] = 0;
            break;
    
        case 1:	//normal note
            switch(i % 7)
            {
            case 0:
                noteweight[i-MINPITCH] = 10;
                break;
            case 2:
                noteweight[i-MINPITCH] = 9;
                break;
            case 4:
                noteweight[i-MINPITCH] = 8;
                break;
            default:
                noteweight[i-MINPITCH] = 0;
                break;
            }
            break;
    
        case 2:	//leaning note
            switch(i % 7)
            {
            case 1:
                noteweight[i-MINPITCH] = 10;
                break;
            case 3:
                noteweight[i-MINPITCH] = 10;
                break;
            case 5:
                noteweight[i-MINPITCH] = 10;
                break;
            case 6:
                noteweight[i-MINPITCH] = 10;
            default:
                noteweight[i-MINPITCH] = 0;
                break;
            }
            break;
    
        case 3:	//passing note
            noteweight[i-MINPITCH] = 10;
            break;
        }
    }
    
    noteweight[i-MINPITCH] = (randomfactor * Math.random()) + ((1-randomfactor) * noteweight[i-MINPITCH]);

}


function findpitch()
{
    //Make an array of probabilities
    var probs = new Array();
    for(i=MINPITCH; i<MAXPITCH; i++)
    {
        var totalprob = rangeweight[i-MINPITCH] * noteweight[i-MINPITCH];
        for(j=0; j<totalprob; j++)
        {
            probs[probs.length] = i;
        }
    }
    
    //Pick one element from probability array
    var rand = Math.floor(Math.random() * probs.length);
    
    //Set chosen number
    pitch = probs[rand];
    
}
